# ---------------------- load/install packages ----------------------

if (!("gridExtra" %in% rownames(installed.packages()))) {
  install.packages("gridExtra", repos="http://cran.xl-mirror.nl/") 
}
library(gridExtra)
if (!("ggplot2" %in% rownames(installed.packages()))) {
  install.packages("ggplot2", repos="http://cran.xl-mirror.nl/") 
}
library(ggplot2)
if (!("plyr" %in% rownames(installed.packages()))) {
  install.packages("plyr", repos="http://cran.xl-mirror.nl/") 
}			
library(plyr)

if (!("data.table" %in% rownames(installed.packages()))) {
  install.packages("data.table", repos="http://cran.xl-mirror.nl/") 
}
library(data.table)

if (!("reshape2" %in% rownames(installed.packages()))) {
  install.packages("reshape2", repos="http://cran.xl-mirror.nl/") 
}
library(reshape2)

if (!("lymphclon" %in% rownames(installed.packages()))) {
  install.packages("lymphclon", repos="http://cran.xl-mirror.nl/") 
}
library(lymphclon)

# ---------------------- parameters ----------------------

args <- commandArgs(trailingOnly = TRUE)

infile = args[1] #path to input file
outfile = args[2] #path to output file
outdir = args[3] #path to output folder (html/images/data)
clonaltype = args[4] #clonaltype definition, or 'none' for no unique filtering
ct = unlist(strsplit(clonaltype, ","))
species = args[5] #human or mouse
locus = args[6] # IGH, IGK, IGL, TRB, TRA, TRG or TRD
filterproductive = ifelse(args[7] == "yes", T, F) #should unproductive sequences be filtered out? (yes/no)
clonality_method = args[8]

# ---------------------- Data preperation ----------------------

inputdata = read.table(infile, sep="\t", header=TRUE, fill=T, comment.char="")

setwd(outdir)

# remove weird rows
inputdata = inputdata[inputdata$Sample != "",]

#remove the allele from the V,D and J genes
inputdata$Top.V.Gene = gsub("[*]([0-9]+)", "", inputdata$Top.V.Gene)
inputdata$Top.D.Gene = gsub("[*]([0-9]+)", "", inputdata$Top.D.Gene)
inputdata$Top.J.Gene = gsub("[*]([0-9]+)", "", inputdata$Top.J.Gene)

inputdata$clonaltype = 1:nrow(inputdata)

PRODF = inputdata
UNPROD = inputdata
if(filterproductive){
  if("Functionality" %in% colnames(inputdata)) { # "Functionality" is an IMGT column
    PRODF = inputdata[inputdata$Functionality == "productive" | inputdata$Functionality == "productive (see comment)", ]
    UNPROD = inputdata[!(inputdata$Functionality == "productive" | inputdata$Functionality == "productive (see comment)"), ]
  } else {
    PRODF = inputdata[inputdata$VDJ.Frame != "In-frame with stop codon" & inputdata$VDJ.Frame != "Out-of-frame" & inputdata$CDR3.Found.How != "NOT_FOUND" , ]
    UNPROD = inputdata[!(inputdata$VDJ.Frame != "In-frame with stop codon" & inputdata$VDJ.Frame != "Out-of-frame" & inputdata$CDR3.Found.How != "NOT_FOUND" ), ]
  }
}

clonalityFrame = PRODF

#remove duplicates based on the clonaltype
if(clonaltype != "none"){
  clonaltype = paste(clonaltype, ",Sample", sep="") #add sample column to clonaltype, unique within samples
  PRODF$clonaltype = do.call(paste, c(PRODF[unlist(strsplit(clonaltype, ","))], sep = ":"))
  PRODF = PRODF[!duplicated(PRODF$clonaltype), ]
  
  UNPROD$clonaltype = do.call(paste, c(UNPROD[unlist(strsplit(clonaltype, ","))], sep = ":"))
  UNPROD = UNPROD[!duplicated(UNPROD$clonaltype), ]
  
  #again for clonalityFrame but with sample+replicate
  clonalityFrame$clonaltype = do.call(paste, c(clonalityFrame[unlist(strsplit(clonaltype, ","))], sep = ":"))
  clonalityFrame$clonality_clonaltype = do.call(paste, c(clonalityFrame[unlist(strsplit(paste(clonaltype, ",Replicate", sep=""), ","))], sep = ":"))
  clonalityFrame = clonalityFrame[!duplicated(clonalityFrame$clonality_clonaltype), ]
}

PRODF$freq = 1

if(any(grepl(pattern="_", x=PRODF$ID))){ #the frequency can be stored in the ID with the pattern ".*_freq_.*"
  PRODF$freq = gsub("^[0-9]+_", "", PRODF$ID)
  PRODF$freq = gsub("_.*", "", PRODF$freq)
  PRODF$freq = as.numeric(PRODF$freq)
  if(any(is.na(PRODF$freq))){ #if there was an "_" in the ID, but not the frequency, go back to frequency of 1 for every sequence
    PRODF$freq = 1
  }
}



#write the complete dataset that is left over, will be the input if 'none' for clonaltype and 'no' for filterproductive
write.table(PRODF, "allUnique.txt", sep=",",quote=F,row.names=F,col.names=T)
write.table(PRODF, "allUnique.csv", sep="\t",quote=F,row.names=F,col.names=T)
write.table(UNPROD, "allUnproductive.csv", sep=",",quote=F,row.names=F,col.names=T)

#write the samples to a file
sampleFile <- file("samples.txt")
un = unique(inputdata$Sample)
un = paste(un, sep="\n")
writeLines(un, sampleFile)
close(sampleFile)

# ---------------------- Counting the productive/unproductive and unique sequences ----------------------

inputdata.dt = data.table(inputdata) #for speed

if(clonaltype == "none"){
  ct = c("clonaltype")
}

inputdata.dt$samples_replicates = paste(inputdata.dt$Sample, inputdata.dt$Replicate, sep="_")
samples_replicates = c(unique(inputdata.dt$samples_replicates), unique(as.character(inputdata.dt$Sample)))
frequency_table = data.frame(ID = samples_replicates[order(samples_replicates)])


sample_productive_count = inputdata.dt[, list(All=.N, 
                                              Productive = nrow(.SD[.SD$Functionality == "productive" | .SD$Functionality == "productive (see comment)",]), 
                                              perc_prod = 1,
                                              Productive_unique = nrow(.SD[.SD$Functionality == "productive" | .SD$Functionality == "productive (see comment)",list(count=.N),by=ct]), 
                                              perc_prod_un = 1,
                                              Unproductive= nrow(.SD[.SD$Functionality != "productive" & .SD$Functionality != "productive (see comment)",]),
                                              perc_unprod = 1,
                                              Unproductive_unique =nrow(.SD[.SD$Functionality != "productive" & .SD$Functionality != "productive (see comment)",list(count=.N),by=ct]),
                                              perc_unprod_un = 1),
                                       by=c("Sample")]

sample_productive_count$perc_prod = round(sample_productive_count$Productive / sample_productive_count$All * 100)
sample_productive_count$perc_prod_un = round(sample_productive_count$Productive_unique / sample_productive_count$All * 100)

sample_productive_count$perc_unprod = round(sample_productive_count$Unproductive / sample_productive_count$All * 100)
sample_productive_count$perc_unprod_un = round(sample_productive_count$Unproductive_unique / sample_productive_count$All * 100)


sample_replicate_productive_count = inputdata.dt[, list(All=.N, 
                                                        Productive = nrow(.SD[.SD$Functionality == "productive" | .SD$Functionality == "productive (see comment)",]), 
                                                        perc_prod = 1,
                                                        Productive_unique = nrow(.SD[.SD$Functionality == "productive" | .SD$Functionality == "productive (see comment)",list(count=.N),by=ct]), 
                                                        perc_prod_un = 1,
                                                        Unproductive= nrow(.SD[.SD$Functionality != "productive" & .SD$Functionality != "productive (see comment)",]),
                                                        perc_unprod = 1,
                                                        Unproductive_unique =nrow(.SD[.SD$Functionality != "productive" & .SD$Functionality != "productive (see comment)",list(count=.N),by=ct]),
                                                        perc_unprod_un = 1),
                                                 by=c("samples_replicates")]

sample_replicate_productive_count$perc_prod = round(sample_replicate_productive_count$Productive / sample_replicate_productive_count$All * 100)
sample_replicate_productive_count$perc_prod_un = round(sample_replicate_productive_count$Productive_unique / sample_replicate_productive_count$All * 100)

sample_replicate_productive_count$perc_unprod = round(sample_replicate_productive_count$Unproductive / sample_replicate_productive_count$All * 100)
sample_replicate_productive_count$perc_unprod_un = round(sample_replicate_productive_count$Unproductive_unique / sample_replicate_productive_count$All * 100)

setnames(sample_replicate_productive_count, colnames(sample_productive_count))

counts = rbind(sample_replicate_productive_count, sample_productive_count)
counts = counts[order(counts$Sample),]

write.table(x=counts, file="productive_counting.txt", sep=",",quote=F,row.names=F,col.names=F)

# ---------------------- Frequency calculation for V, D and J ----------------------

PRODFV = data.frame(data.table(PRODF)[, list(Length=sum(freq)), by=c("Sample", "Top.V.Gene")])
Total = ddply(PRODFV, .(Sample), function(x) data.frame(Total = sum(x$Length)))
PRODFV = merge(PRODFV, Total, by.x='Sample', by.y='Sample', all.x=TRUE)
PRODFV = ddply(PRODFV, c("Sample", "Top.V.Gene"), summarise, relFreq= (Length*100 / Total))

PRODFD = data.frame(data.table(PRODF)[, list(Length=sum(freq)), by=c("Sample", "Top.D.Gene")])
Total = ddply(PRODFD, .(Sample), function(x) data.frame(Total = sum(x$Length)))
PRODFD = merge(PRODFD, Total, by.x='Sample', by.y='Sample', all.x=TRUE)
PRODFD = ddply(PRODFD, c("Sample", "Top.D.Gene"), summarise, relFreq= (Length*100 / Total))

PRODFJ = data.frame(data.table(PRODF)[, list(Length=sum(freq)), by=c("Sample", "Top.J.Gene")])
Total = ddply(PRODFJ, .(Sample), function(x) data.frame(Total = sum(x$Length)))
PRODFJ = merge(PRODFJ, Total, by.x='Sample', by.y='Sample', all.x=TRUE)
PRODFJ = ddply(PRODFJ, c("Sample", "Top.J.Gene"), summarise, relFreq= (Length*100 / Total))

# ---------------------- Setting up the gene names for the different species/loci ----------------------

Vchain = ""
Dchain = ""
Jchain = ""

if(species == "custom"){
	print("Custom genes: ")
	splt = unlist(strsplit(locus, ";"))
	print(paste("V:", splt[1]))
	print(paste("D:", splt[2]))
	print(paste("J:", splt[3]))
	
	Vchain = unlist(strsplit(splt[1], ","))
	Vchain = data.frame(v.name = Vchain, chr.orderV = 1:length(Vchain))
	
	Dchain = unlist(strsplit(splt[2], ","))
	if(length(Dchain) > 0){
		Dchain = data.frame(v.name = Dchain, chr.orderD = 1:length(Dchain))
	} else {
		Dchain = data.frame(v.name = character(0), chr.orderD = numeric(0))
	}
	
	Jchain = unlist(strsplit(splt[3], ","))
	Jchain = data.frame(v.name = Jchain, chr.orderJ = 1:length(Jchain))

} else {
	genes = read.table("genes.txt", sep="\t", header=TRUE, fill=T, comment.char="")

	Vchain = genes[grepl(species, genes$Species) & genes$locus == locus & genes$region == "V",c("IMGT.GENE.DB", "chr.order")]
	colnames(Vchain) = c("v.name", "chr.orderV")
	Dchain = genes[grepl(species, genes$Species) & genes$locus == locus & genes$region == "D",c("IMGT.GENE.DB", "chr.order")]
	colnames(Dchain) = c("v.name", "chr.orderD")
	Jchain = genes[grepl(species, genes$Species) & genes$locus == locus & genes$region == "J",c("IMGT.GENE.DB", "chr.order")]
	colnames(Jchain) = c("v.name", "chr.orderJ")
}
useD = TRUE
if(nrow(Dchain) == 0){
  useD = FALSE
  cat("No D Genes in this species/locus")
}
print(paste("useD:", useD))

# ---------------------- merge with the frequency count ----------------------

PRODFV = merge(PRODFV, Vchain, by.x='Top.V.Gene', by.y='v.name', all.x=TRUE)

PRODFD = merge(PRODFD, Dchain, by.x='Top.D.Gene', by.y='v.name', all.x=TRUE)

PRODFJ = merge(PRODFJ, Jchain, by.x='Top.J.Gene', by.y='v.name', all.x=TRUE)

# ---------------------- Create the V, D and J frequency plots and write the data.frame for every plot to a file ----------------------

pV = ggplot(PRODFV)
pV = pV + geom_bar( aes( x=factor(reorder(Top.V.Gene, chr.orderV)), y=relFreq, fill=Sample), stat='identity', position="dodge") + theme(axis.text.x = element_text(angle = 90, hjust = 1))
pV = pV + xlab("Summary of V gene") + ylab("Frequency") + ggtitle("Relative frequency of V gene usage")
write.table(x=PRODFV, file="VFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

png("VPlot.png",width = 1280, height = 720)
pV
dev.off();

if(useD){
  pD = ggplot(PRODFD)
  pD = pD + geom_bar( aes( x=factor(reorder(Top.D.Gene, chr.orderD)), y=relFreq, fill=Sample), stat='identity', position="dodge") + theme(axis.text.x = element_text(angle = 90, hjust = 1))
  pD = pD + xlab("Summary of D gene") + ylab("Frequency") + ggtitle("Relative frequency of D gene usage")
  write.table(x=PRODFD, file="DFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)
  
  png("DPlot.png",width = 800, height = 600)
  print(pD)
  dev.off();
}

pJ = ggplot(PRODFJ)
pJ = pJ + geom_bar( aes( x=factor(reorder(Top.J.Gene, chr.orderJ)), y=relFreq, fill=Sample), stat='identity', position="dodge") + theme(axis.text.x = element_text(angle = 90, hjust = 1))
pJ = pJ + xlab("Summary of J gene") + ylab("Frequency") + ggtitle("Relative frequency of J gene usage")
write.table(x=PRODFJ, file="JFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

png("JPlot.png",width = 800, height = 600)
pJ
dev.off();

pJ = ggplot(PRODFJ)
pJ = pJ + geom_bar( aes( x=factor(reorder(Top.J.Gene, chr.orderJ)), y=relFreq, fill=Sample), stat='identity', position="dodge") + theme(axis.text.x = element_text(angle = 90, hjust = 1))
pJ = pJ + xlab("Summary of J gene") + ylab("Frequency") + ggtitle("Relative frequency of J gene usage")
write.table(x=PRODFJ, file="JFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

png("JPlot.png",width = 800, height = 600)
pJ
dev.off();

# ---------------------- Now the frequency plots of the V, D and J families ----------------------

VGenes = PRODF[,c("Sample", "Top.V.Gene")]
VGenes$Top.V.Gene = gsub("-.*", "", VGenes$Top.V.Gene)
VGenes = data.frame(data.table(VGenes)[, list(Count=.N), by=c("Sample", "Top.V.Gene")])
TotalPerSample = data.frame(data.table(VGenes)[, list(total=sum(.SD$Count)), by=Sample])
VGenes = merge(VGenes, TotalPerSample, by="Sample")
VGenes$Frequency = VGenes$Count * 100 / VGenes$total
VPlot = ggplot(VGenes)
VPlot = VPlot + geom_bar(aes( x = Top.V.Gene, y = Frequency, fill = Sample), stat='identity', position='dodge' ) + theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
  ggtitle("Distribution of V gene families") + 
  ylab("Percentage of sequences")
png("VFPlot.png")
VPlot
dev.off();
write.table(x=VGenes, file="VFFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

if(useD){
  DGenes = PRODF[,c("Sample", "Top.D.Gene")]
  DGenes$Top.D.Gene = gsub("-.*", "", DGenes$Top.D.Gene)
  DGenes = data.frame(data.table(DGenes)[, list(Count=.N), by=c("Sample", "Top.D.Gene")])
  TotalPerSample = data.frame(data.table(DGenes)[, list(total=sum(.SD$Count)), by=Sample])
  DGenes = merge(DGenes, TotalPerSample, by="Sample")
  DGenes$Frequency = DGenes$Count * 100 / DGenes$total
  DPlot = ggplot(DGenes)
  DPlot = DPlot + geom_bar(aes( x = Top.D.Gene, y = Frequency, fill = Sample), stat='identity', position='dodge' ) + theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
    ggtitle("Distribution of D gene families") + 
    ylab("Percentage of sequences")
  png("DFPlot.png")
  print(DPlot)
  dev.off();
  write.table(x=DGenes, file="DFFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)
}

JGenes = PRODF[,c("Sample", "Top.J.Gene")]
JGenes$Top.J.Gene = gsub("-.*", "", JGenes$Top.J.Gene)
JGenes = data.frame(data.table(JGenes)[, list(Count=.N), by=c("Sample", "Top.J.Gene")])
TotalPerSample = data.frame(data.table(JGenes)[, list(total=sum(.SD$Count)), by=Sample])
JGenes = merge(JGenes, TotalPerSample, by="Sample")
JGenes$Frequency = JGenes$Count * 100 / JGenes$total
JPlot = ggplot(JGenes)
JPlot = JPlot + geom_bar(aes( x = Top.J.Gene, y = Frequency, fill = Sample), stat='identity', position='dodge' ) + theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
  ggtitle("Distribution of J gene families") + 
  ylab("Percentage of sequences")
png("JFPlot.png")
JPlot
dev.off();
write.table(x=JGenes, file="JFFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

# ---------------------- Plotting the cdr3 length ----------------------

CDR3Length = data.frame(data.table(PRODF)[, list(Count=.N), by=c("Sample", "CDR3.Length.DNA")])
TotalPerSample = data.frame(data.table(CDR3Length)[, list(total=sum(.SD$Count)), by=Sample])
CDR3Length = merge(CDR3Length, TotalPerSample, by="Sample")
CDR3Length$Frequency = CDR3Length$Count * 100 / CDR3Length$total
CDR3LengthPlot = ggplot(CDR3Length)
CDR3LengthPlot = CDR3LengthPlot + geom_bar(aes( x = CDR3.Length.DNA, y = Frequency, fill = Sample), stat='identity', position='dodge' ) + theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
  ggtitle("Length distribution of CDR3") + 
  xlab("CDR3 Length") + 
  ylab("Percentage of sequences")
png("CDR3LengthPlot.png",width = 1280, height = 720)
CDR3LengthPlot
dev.off()
write.table(x=CDR3Length, file="CDR3LengthPlot.csv", sep=",",quote=F,row.names=F,col.names=T)

# ---------------------- Plot the heatmaps ----------------------


#get the reverse order for the V and D genes
revVchain = Vchain
revDchain = Dchain
revVchain$chr.orderV = rev(revVchain$chr.orderV)
revDchain$chr.orderD = rev(revDchain$chr.orderD)

if(useD){
  plotVD <- function(dat){
    if(length(dat[,1]) == 0){
      return()
    }
    img = ggplot() + 
      geom_tile(data=dat, aes(x=factor(reorder(Top.D.Gene, chr.orderD)), y=factor(reorder(Top.V.Gene, chr.orderV)), fill=relLength)) + 
      theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
      scale_fill_gradient(low="gold", high="blue", na.value="white") + 
      ggtitle(paste(unique(dat$Sample), " (N=" , sum(dat$Length, na.rm=T) ,")", sep="")) + 
      xlab("D genes") + 
      ylab("V Genes")
    
    png(paste("HeatmapVD_", unique(dat[3])[1,1] , ".png", sep=""), width=150+(15*length(Dchain$v.name)), height=100+(15*length(Vchain$v.name)))
    print(img)
    dev.off()
    write.table(x=acast(dat, Top.V.Gene~Top.D.Gene, value.var="Length"), file=paste("HeatmapVD_", unique(dat[3])[1,1], ".csv", sep=""), sep=",",quote=F,row.names=T,col.names=NA)
  }
  
  VandDCount = data.frame(data.table(PRODF)[, list(Length=.N), by=c("Top.V.Gene", "Top.D.Gene", "Sample")])
  
  VandDCount$l = log(VandDCount$Length)
  maxVD = data.frame(data.table(VandDCount)[, list(max=max(l)), by=c("Sample")])
  VandDCount = merge(VandDCount, maxVD, by.x="Sample", by.y="Sample", all.x=T)
  VandDCount$relLength = VandDCount$l / VandDCount$max
  
  cartegianProductVD = expand.grid(Top.V.Gene = Vchain$v.name, Top.D.Gene = Dchain$v.name, Sample = unique(inputdata$Sample))
  
  completeVD = merge(VandDCount, cartegianProductVD, all.y=TRUE)
  completeVD = merge(completeVD, revVchain, by.x="Top.V.Gene", by.y="v.name", all.x=TRUE)
  completeVD = merge(completeVD, Dchain, by.x="Top.D.Gene", by.y="v.name", all.x=TRUE)
  VDList = split(completeVD, f=completeVD[,"Sample"])
  
  lapply(VDList, FUN=plotVD)
}

plotVJ <- function(dat){
  if(length(dat[,1]) == 0){
    return()
  }
  cat(paste(unique(dat[3])[1,1]))
  img = ggplot() + 
    geom_tile(data=dat, aes(x=factor(reorder(Top.J.Gene, chr.orderJ)), y=factor(reorder(Top.V.Gene, chr.orderV)), fill=relLength)) + 
    theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
    scale_fill_gradient(low="gold", high="blue", na.value="white") + 
    ggtitle(paste(unique(dat$Sample), " (N=" , sum(dat$Length, na.rm=T) ,")", sep="")) + 
    xlab("J genes") + 
    ylab("V Genes")
  
  png(paste("HeatmapVJ_", unique(dat[3])[1,1] , ".png", sep=""), width=150+(15*length(Jchain$v.name)), height=100+(15*length(Vchain$v.name)))
  print(img)
  dev.off()
  write.table(x=acast(dat, Top.V.Gene~Top.J.Gene, value.var="Length"), file=paste("HeatmapVJ_", unique(dat[3])[1,1], ".csv", sep=""), sep=",",quote=F,row.names=T,col.names=NA)
}

VandJCount = data.frame(data.table(PRODF)[, list(Length=.N), by=c("Top.V.Gene", "Top.J.Gene", "Sample")])

VandJCount$l = log(VandJCount$Length)
maxVJ = data.frame(data.table(VandJCount)[, list(max=max(l)), by=c("Sample")])
VandJCount = merge(VandJCount, maxVJ, by.x="Sample", by.y="Sample", all.x=T)
VandJCount$relLength = VandJCount$l / VandJCount$max

cartegianProductVJ = expand.grid(Top.V.Gene = Vchain$v.name, Top.J.Gene = Jchain$v.name, Sample = unique(inputdata$Sample))

completeVJ = merge(VandJCount, cartegianProductVJ, all.y=TRUE)
completeVJ = merge(completeVJ, revVchain, by.x="Top.V.Gene", by.y="v.name", all.x=TRUE)
completeVJ = merge(completeVJ, Jchain, by.x="Top.J.Gene", by.y="v.name", all.x=TRUE)
VJList = split(completeVJ, f=completeVJ[,"Sample"])
lapply(VJList, FUN=plotVJ)

if(useD){
  plotDJ <- function(dat){
    if(length(dat[,1]) == 0){
      return()
    }
    img = ggplot() + 
      geom_tile(data=dat, aes(x=factor(reorder(Top.J.Gene, chr.orderJ)), y=factor(reorder(Top.D.Gene, chr.orderD)), fill=relLength)) + 
      theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
      scale_fill_gradient(low="gold", high="blue", na.value="white") + 
      ggtitle(paste(unique(dat$Sample), " (N=" , sum(dat$Length, na.rm=T) ,")", sep="")) + 
      xlab("J genes") + 
      ylab("D Genes")
    
    png(paste("HeatmapDJ_", unique(dat[3])[1,1] , ".png", sep=""), width=150+(15*length(Jchain$v.name)), height=100+(15*length(Dchain$v.name)))
    print(img)
    dev.off()
    write.table(x=acast(dat, Top.D.Gene~Top.J.Gene, value.var="Length"), file=paste("HeatmapDJ_", unique(dat[3])[1,1], ".csv", sep=""), sep=",",quote=F,row.names=T,col.names=NA)
  }
  
  
  DandJCount = data.frame(data.table(PRODF)[, list(Length=.N), by=c("Top.D.Gene", "Top.J.Gene", "Sample")])
  
  DandJCount$l = log(DandJCount$Length)
  maxDJ = data.frame(data.table(DandJCount)[, list(max=max(l)), by=c("Sample")])
  DandJCount = merge(DandJCount, maxDJ, by.x="Sample", by.y="Sample", all.x=T)
  DandJCount$relLength = DandJCount$l / DandJCount$max
  
  cartegianProductDJ = expand.grid(Top.D.Gene = Dchain$v.name, Top.J.Gene = Jchain$v.name, Sample = unique(inputdata$Sample))
  
  completeDJ = merge(DandJCount, cartegianProductDJ, all.y=TRUE)
  completeDJ = merge(completeDJ, revDchain, by.x="Top.D.Gene", by.y="v.name", all.x=TRUE)
  completeDJ = merge(completeDJ, Jchain, by.x="Top.J.Gene", by.y="v.name", all.x=TRUE)
  DJList = split(completeDJ, f=completeDJ[,"Sample"])
  lapply(DJList, FUN=plotDJ)
}


# ---------------------- calculating the clonality score ----------------------

if("Replicate" %in% colnames(inputdata)) #can only calculate clonality score when replicate information is available
{
  if(clonality_method == "boyd"){
    samples = split(clonalityFrame, clonalityFrame$Sample, drop=T)
   
    for (sample in samples){
      res = data.frame(paste=character(0))
      sample_id = unique(sample$Sample)[[1]]
      for(replicate in unique(sample$Replicate)){
        tmp = sample[sample$Replicate == replicate,]
        clone_table = data.frame(table(tmp$clonaltype))
        clone_col_name = paste("V", replicate, sep="")
        colnames(clone_table) = c("paste", clone_col_name)
        res = merge(res, clone_table, by="paste", all=T)
      }
      
      res[is.na(res)] = 0      
      infer.result = infer.clonality(as.matrix(res[,2:ncol(res)]))
      
      write.table(data.table(infer.result[[12]]), file=paste("lymphclon_clonality_", sample_id, ".csv", sep=""), sep=",",quote=F,row.names=F,col.names=F)
      
      res$type = rowSums(res[,2:ncol(res)])
      
      coincidence.table = data.frame(table(res$type))
      colnames(coincidence.table) = c("Coincidence Type",  "Raw Coincidence Freq")
      write.table(coincidence.table, file=paste("lymphclon_coincidences_", sample_id, ".csv", sep=""), sep=",",quote=F,row.names=F,col.names=T)
    }
  } else {
    write.table(clonalityFrame, "clonalityComplete.csv", sep=",",quote=F,row.names=F,col.names=T)
      
    clonalFreq = data.frame(data.table(clonalityFrame)[, list(Type=.N), by=c("Sample", "clonaltype")])
    clonalFreqCount = data.frame(data.table(clonalFreq)[, list(Count=.N), by=c("Sample", "Type")])
    clonalFreqCount$realCount = clonalFreqCount$Type * clonalFreqCount$Count
    clonalSum = data.frame(data.table(clonalFreqCount)[, list(Reads=sum(realCount)), by=c("Sample")])
    clonalFreqCount = merge(clonalFreqCount, clonalSum, by.x="Sample", by.y="Sample")
    
    ct = c('Type\tWeight\n2\t1\n3\t3\n4\t6\n5\t10\n6\t15')
    tcct = textConnection(ct)
    CT  = read.table(tcct, sep="\t", header=TRUE)
    close(tcct)
    clonalFreqCount = merge(clonalFreqCount, CT, by.x="Type", by.y="Type", all.x=T)
    clonalFreqCount$WeightedCount = clonalFreqCount$Count * clonalFreqCount$Weight
    
    ReplicateReads = data.frame(data.table(clonalityFrame)[, list(Type=.N), by=c("Sample", "Replicate", "clonaltype")])
    ReplicateReads = data.frame(data.table(ReplicateReads)[, list(Reads=.N), by=c("Sample", "Replicate")])
    clonalFreqCount$Reads = as.numeric(clonalFreqCount$Reads)
    ReplicateReads$squared = ReplicateReads$Reads * ReplicateReads$Reads
    
    ReplicatePrint <- function(dat){
      write.table(dat[-1], paste("ReplicateReads_", unique(dat[1])[1,1] , ".csv", sep=""), sep=",",quote=F,na="-",row.names=F,col.names=F)
    }
    
    ReplicateSplit = split(ReplicateReads, f=ReplicateReads[,"Sample"])
    lapply(ReplicateSplit, FUN=ReplicatePrint)
    
    ReplicateReads = data.frame(data.table(ReplicateReads)[, list(ReadsSum=sum(as.numeric(Reads)), ReadsSquaredSum=sum(as.numeric(squared))), by=c("Sample")])
    clonalFreqCount = merge(clonalFreqCount, ReplicateReads, by.x="Sample", by.y="Sample", all.x=T)
    
    ReplicateSumPrint <- function(dat){
      write.table(dat[-1], paste("ReplicateSumReads_", unique(dat[1])[1,1] , ".csv", sep=""), sep=",",quote=F,na="-",row.names=F,col.names=F)
    }
    
    ReplicateSumSplit = split(ReplicateReads, f=ReplicateReads[,"Sample"])
    lapply(ReplicateSumSplit, FUN=ReplicateSumPrint)
    
    clonalFreqCountSum = data.frame(data.table(clonalFreqCount)[, list(Numerator=sum(WeightedCount, na.rm=T)), by=c("Sample")])
    clonalFreqCount = merge(clonalFreqCount, clonalFreqCountSum, by.x="Sample", by.y="Sample", all.x=T)
    clonalFreqCount$ReadsSum = as.numeric(clonalFreqCount$ReadsSum) #prevent integer overflow
    clonalFreqCount$Denominator = (((clonalFreqCount$ReadsSum * clonalFreqCount$ReadsSum) - clonalFreqCount$ReadsSquaredSum) / 2)
    clonalFreqCount$Result = (clonalFreqCount$Numerator + 1) / (clonalFreqCount$Denominator + 1)
    
    ClonalityScorePrint <- function(dat){
      write.table(dat$Result, paste("ClonalityScore_", unique(dat[1])[1,1] , ".csv", sep=""), sep=",",quote=F,na="-",row.names=F,col.names=F)
    }
    
    clonalityScore = clonalFreqCount[c("Sample", "Result")]
    clonalityScore = unique(clonalityScore)
    
    clonalityScoreSplit = split(clonalityScore, f=clonalityScore[,"Sample"])
    lapply(clonalityScoreSplit, FUN=ClonalityScorePrint)
    
    clonalityOverview = clonalFreqCount[c("Sample", "Type", "Count", "Weight", "WeightedCount")]
    
    
    
    ClonalityOverviewPrint <- function(dat){
      write.table(dat[-1], paste("ClonalityOverView_", unique(dat[1])[1,1] , ".csv", sep=""), sep=",",quote=F,na="-",row.names=F,col.names=F)
    }
    
    clonalityOverviewSplit = split(clonalityOverview, f=clonalityOverview$Sample)
    lapply(clonalityOverviewSplit, FUN=ClonalityOverviewPrint)
  }
}

imgtcolumns = c("X3V.REGION.trimmed.nt.nb","P3V.nt.nb", "N1.REGION.nt.nb", "P5D.nt.nb", "X5D.REGION.trimmed.nt.nb", "X3D.REGION.trimmed.nt.nb", "P3D.nt.nb", "N2.REGION.nt.nb", "P5J.nt.nb", "X5J.REGION.trimmed.nt.nb", "X3V.REGION.trimmed.nt.nb", "X5D.REGION.trimmed.nt.nb", "X3D.REGION.trimmed.nt.nb", "X5J.REGION.trimmed.nt.nb", "N1.REGION.nt.nb", "N2.REGION.nt.nb", "P3V.nt.nb", "P5D.nt.nb", "P3D.nt.nb", "P5J.nt.nb")
if(all(imgtcolumns %in% colnames(inputdata)))
{
	print("MEAN P3V.nt.nb:")
	print(PRODF$P3V.nt.nb)
	print(mean(PRODF$P3V.nt.nb, na.rm=T))
	print(head(PRODF))
  newData = data.frame(data.table(PRODF)[,list(unique=.N, 
                                               VH.DEL=mean(.SD$X3V.REGION.trimmed.nt.nb, na.rm=T),
                                               P1=mean(.SD$P3V.nt.nb, na.rm=T),
                                               N1=mean(.SD$N1.REGION.nt.nb, na.rm=T),
                                               P2=mean(.SD$P5D.nt.nb, na.rm=T),
                                               DEL.DH=mean(.SD$X5D.REGION.trimmed.nt.nb, na.rm=T),
                                               DH.DEL=mean(.SD$X3D.REGION.trimmed.nt.nb, na.rm=T),
                                               P3=mean(.SD$P3D.nt.nb, na.rm=T),
                                               N2=mean(.SD$N2.REGION.nt.nb, na.rm=T),
                                               P4=mean(.SD$P5J.nt.nb, na.rm=T),
                                               DEL.JH=mean(.SD$X5J.REGION.trimmed.nt.nb, na.rm=T),
                                               Total.Del=(	mean(.SD$X3V.REGION.trimmed.nt.nb, na.rm=T) + 
                                                             mean(.SD$X5D.REGION.trimmed.nt.nb, na.rm=T) + 
                                                             mean(.SD$X3D.REGION.trimmed.nt.nb, na.rm=T) +
                                                             mean(.SD$X5J.REGION.trimmed.nt.nb, na.rm=T)),
                                               
                                               Total.N=(	mean(.SD$N1.REGION.nt.nb, na.rm=T) +
                                                           mean(.SD$N2.REGION.nt.nb, na.rm=T)),
                                               
                                               Total.P=(	mean(.SD$P3V.nt.nb, na.rm=T) +
                                                           mean(.SD$P5D.nt.nb, na.rm=T) +
                                                           mean(.SD$P3D.nt.nb, na.rm=T) +
                                                           mean(.SD$P5J.nt.nb, na.rm=T))),
                                         by=c("Sample")])
  write.table(newData, "junctionAnalysisProd.csv" , sep=",",quote=F,na="-",row.names=F,col.names=F)
  
  newData = data.frame(data.table(UNPROD)[,list(unique=.N, 
                                                VH.DEL=mean(.SD$X3V.REGION.trimmed.nt.nb, na.rm=T),
                                                P1=mean(.SD$P3V.nt.nb, na.rm=T),
                                                N1=mean(.SD$N1.REGION.nt.nb, na.rm=T),
                                                P2=mean(.SD$P5D.nt.nb, na.rm=T),
                                                DEL.DH=mean(.SD$X5D.REGION.trimmed.nt.nb, na.rm=T),
                                                DH.DEL=mean(.SD$X3D.REGION.trimmed.nt.nb, na.rm=T),
                                                P3=mean(.SD$P3D.nt.nb, na.rm=T),
                                                N2=mean(.SD$N2.REGION.nt.nb, na.rm=T),
                                                P4=mean(.SD$P5J.nt.nb, na.rm=T),
                                                DEL.JH=mean(.SD$X5J.REGION.trimmed.nt.nb, na.rm=T),
                                                Total.Del=(	mean(.SD$X3V.REGION.trimmed.nt.nb, na.rm=T) + 
                                                            mean(.SD$X5D.REGION.trimmed.nt.nb, na.rm=T) + 
                                                            mean(.SD$X3D.REGION.trimmed.nt.nb, na.rm=T) +
                                                            mean(.SD$X5J.REGION.trimmed.nt.nb, na.rm=T)),
                                                
                                                Total.N=(	mean(.SD$N1.REGION.nt.nb, na.rm=T) +
                                                          mean(.SD$N2.REGION.nt.nb, na.rm=T)),
                                                
																								Total.P=(	mean(.SD$P3V.nt.nb, na.rm=T) +
																													mean(.SD$P5D.nt.nb, na.rm=T) +
																													mean(.SD$P3D.nt.nb, na.rm=T) +
																													mean(.SD$P5J.nt.nb, na.rm=T))),
                                          by=c("Sample")])
  write.table(newData, "junctionAnalysisUnProd.csv" , sep=",",quote=F,na="-",row.names=F,col.names=F)
}

# ---------------------- AA composition in CDR3 ----------------------

AACDR3 = PRODF[,c("Sample", "CDR3.Seq")]

TotalPerSample = data.frame(data.table(AACDR3)[, list(total=sum(nchar(as.character(.SD$CDR3.Seq)))), by=Sample])

AAfreq = list()

for(i in 1:nrow(TotalPerSample)){
	sample = TotalPerSample$Sample[i]
  AAfreq[[i]] = data.frame(table(unlist(strsplit(as.character(AACDR3[AACDR3$Sample == sample,c("CDR3.Seq")]), ""))))
  AAfreq[[i]]$Sample = sample
}

AAfreq = ldply(AAfreq, data.frame)
AAfreq = merge(AAfreq, TotalPerSample, by="Sample", all.x = T)
AAfreq$freq_perc = as.numeric(AAfreq$Freq / AAfreq$total * 100)


AAorder = read.table(sep="\t", header=TRUE, text="order.aa\tAA\n1\tR\n2\tK\n3\tN\n4\tD\n5\tQ\n6\tE\n7\tH\n8\tP\n9\tY\n10\tW\n11\tS\n12\tT\n13\tG\n14\tA\n15\tM\n16\tC\n17\tF\n18\tL\n19\tV\n20\tI")
AAfreq = merge(AAfreq, AAorder, by.x='Var1', by.y='AA', all.x=TRUE)

AAfreq = AAfreq[!is.na(AAfreq$order.aa),]

AAfreqplot = ggplot(AAfreq)
AAfreqplot = AAfreqplot + geom_bar(aes( x=factor(reorder(Var1, order.aa)), y = freq_perc, fill = Sample), stat='identity', position='dodge' )
AAfreqplot = AAfreqplot + annotate("rect", xmin = 0.5, xmax = 2.5, ymin = 0, ymax = Inf, fill = "red", alpha = 0.2)
AAfreqplot = AAfreqplot + annotate("rect", xmin = 3.5, xmax = 4.5, ymin = 0, ymax = Inf, fill = "blue", alpha = 0.2)
AAfreqplot = AAfreqplot + annotate("rect", xmin = 5.5, xmax = 6.5, ymin = 0, ymax = Inf, fill = "blue", alpha = 0.2)
AAfreqplot = AAfreqplot + annotate("rect", xmin = 6.5, xmax = 7.5, ymin = 0, ymax = Inf, fill = "red", alpha = 0.2)
AAfreqplot = AAfreqplot + ggtitle("Amino Acid Composition in the CDR3") + xlab("Amino Acid, from Hydrophilic (left) to Hydrophobic (right)") + ylab("Percentage")

png("AAComposition.png",width = 1280, height = 720)
AAfreqplot
dev.off()
write.table(AAfreq, "AAComposition.csv" , sep=",",quote=F,na="-",row.names=F,col.names=T)


